import Axios from "axios";
import ErrorHandling from "./error-handling";

const Error = new ErrorHandling();

// 错误码关键字
const ERROR_KEY = "code";
const getBaseUrl = () => {
  // TODO
  return "";
};
/**
 * axios实例化对象
 * */
class AxiosInstance {
  #Axios = null;
  #CancelToken = Axios.CancelToken;
  #queue = [];

  /**
   * 设置当前请求token
   * @param {string} path
   * @param {string} method
   * @returns {string}
   * */
  #getToken = ({ url, method }) => {
    return `${url}_${method}`;
  };

  /**
   * 添加请求队列
   * */
  #addQueue = (config) => {
    if (process.env.VUE_APP_REQUEST_ASPECT === "false") {
      return null;
    } else {
      return new this.#CancelToken((cancel) => {
        this.#queue.push({
          token: this.#getToken(config),
          cancel,
        });
      });
    }
  };

  /**
   * 移除请求队列
   * */
  #removeQueue = (config) => {
    if (process.env.VUE_APP_REQUEST_ASPECT === "false") {
      return null;
    } else {
      const index = this.#queue.findIndex(
        (q) => q.token === this.#getToken(config)
      );
      if (index > -1) {
        this.#queue[index].cancel();
        this.#queue.splice(index, 1);
      }
    }
  };

  /**
   * 获取baseUrl
   * */
  #getBaseUrl = () => {
    // .env配置文件修改
    return getBaseUrl();
  };

  constructor(config) {
    config = Object.assign(
      {
        timeout: 50 * 1000, // 超时处理
        // responseType: 'json', // 响应数据类型
        withCredentials: true, // 是否允许带cookie这些
        headers: {
          "Content-Type": "application/json",
        },
      },
      config
    );
    this.#Axios = Axios.create(config);

    // 请求拦截
    this.#Axios.interceptors.request.use(
      (config) => {
        this.#removeQueue(config);

        // 增加统一请求前缀
        if (!config.customBaseUrl) {
          config.url = this.#getBaseUrl() + config.url;
        }

        config.cancelToken = this.#addQueue(config);

        // 默认使用 FormData 数据格式
        // if (!Object.prototype.hasOwnProperty.call(config, 'isFormData')) {
        //   config.isFormData = true
        // }

        // 处理数据格式
        if (config.method === "post" && config.isFormData) {
          const formData = new FormData();
          const data = config.data.params || config.data;
          Object.keys(data).forEach((key) => {
            formData.append(key, data[key]);
          });
          config.data = formData;
        }

        return config;
      },
      (error) => {
        this.#removeQueue(error.config);

        console.error("请求失败: ", error && error.data.error.message);

        return Promise.reject(error);
      }
    );

    // 响应拦截
    this.#Axios.interceptors.response.use(
      (response) => {
        this.#removeQueue(response.config);

        if (response.data) {
          if (response.data instanceof Blob) {
            return response.data;
          }
          if (![200].includes(response.data[ERROR_KEY])) {
            const errMsg = Error.business(response.data, ERROR_KEY);
            // 错误处理
            console.error("请求错误: ", errMsg);
            return Promise.reject(response.data);
          }
        }
        if (Object.prototype.hasOwnProperty.call(response, "data")) {
          response = response.data;
        }

        return response;
      },
      (error) => {
        if (error.message) {
          this.#removeQueue(error.config);

          if (error.response) {
            const errMsg = Error.server(error.response);
            console.error("服务器错误: ", errMsg);
          } else {
            console.error("未知错误: ", error.message);
          }
        }

        return Promise.reject(error);
      }
    );

    return this.#Axios;
  }
}

/**
 * api接口
 * 处理常用的请求方法，如有其他需要请自行添加
 * */
class Request {
  constructor(config = {}) {
    this.request = new AxiosInstance(config);
  }

  // 获取token
  static getToken() {
    // TODO
    let token = localStorage.getItem("pro__Access-Token");
    return token ? JSON.parse(token).value : "";
  }

  /**
   * post请求
   * @param {string} url
   * @param {object} [data = {}] data
   * @param {object} [config = {}] config
   * @returns {Promise<resolve, reject>}
   * */
  post(url, data = {}, config = {}) {
    if (!url) {
      throw new Error("请求地址不能为空");
    }

    return this.request.post(
      url,
      data,
      Object.assign(config, {
        headers: {
          "X-Access-Token": Request.getToken(),
        },
      })
    );
  }

  /**
   * get请求
   * @param {string} url
   * @param {object} [data = {}] data
   * @param {object} [config = {}] config
   * @returns {Promise<resolve, reject>}
   * */
  get(url, data = {}, config = {}) {
    if (!url) {
      throw new Error("请求地址不能为空");
    }

    return this.request.get(
      url,
      Object.assign(config, {
        headers: {
          "X-Access-Token": Request.getToken(),
        },
        params: data,
      })
    );
  }

  /**
   * delete请求
   * @param {string} url
   * @param {object} [data = {}] data
   * @param {object} [config = {}] config
   * @returns {Promise<resolve, reject>}
   * */
  delete(url, data = {}, config = {}) {
    if (!url) {
      throw new Error("请求地址不能为空");
    }

    return this.request.delete(
      url,
      Object.assign(config, {
        headers: {
          "X-Access-Token": Request.getToken(),
        },
        params: data,
      })
    );
  }

  /**
   * put请求
   * @param {string} url
   * @param {object} [data = {}] data
   * @param {object} [config = {}] config
   * @returns {Promise<resolve, reject>}
   * */
  put(url, data = {}, config = {}) {
    if (!url) {
      throw new Error("请求地址不能为空");
    }

    return this.request.put(
      url,
      data,
      Object.assign(config, {
        headers: {
          "X-Access-Token": Request.getToken(),
        },
      })
    );
  }

  downFile(url, data = {}, config = {}) {
    if (!url) {
      throw new Error("请求地址不能为空");
    }
    return this.request.get(
      url,
      Object.assign(config, {
        headers: {
          "X-Access-Token": Request.getToken(),
        },
        params: data,
        responseType: "blob",
      })
    );
  }

  /**
   * 获取文件服务访问路径
   * @param avatar
   * @param subStr
   * @returns {*}
   */
  getFileAccessHttpUrl(avatar, subStr) {
    if (!subStr) subStr = "http";
    if (avatar && avatar.startsWith(subStr)) {
      return avatar;
    } else {
      if (avatar && avatar.length > 0 && avatar.indexOf("[") === -1) {
        return window._CONFIG.staticDomainURL + "/" + avatar;
      }
    }
  }

  // TODO 其余 method 请自行添加
}

export { Request, getBaseUrl };

export default new Request();
